@@ -2,7 +2,7 @@
  *  SPDX-FileCopyrightText: 2019 David Redondo <kde@david-redondo.de>
  *  SPDX-FileCopyrightText: 2015 Boudhayan Gupta <bgupta@kde.org>
  *
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "SettingsDialog.h"
@@ -12,10 +12,13 @@
 #include "settings.h"
 #include "ShortcutsOptionsPage.h"
 
+#include <QFontDatabase>
+
 #include <KLocalizedString>
 #include <KShortcutWidget>
 
 
+
 SettingsDialog::SettingsDialog(QWidget *parent) :
     KConfigDialog(parent, QStringLiteral("settings"), Settings::self())
 {
@@ -28,10 +31,16 @@ SettingsDialog::SettingsDialog(QWidget *parent) :
     connect(mShortcutsPage, &ShortcutsOptionsPage::shortCutsChanged, this, [this] {
         updateButtons();
     });
-    resize(600, 590);
     connect(this, &KConfigDialog::currentPageChanged, this, &SettingsDialog::updateButtons);
 }
 
+QSize SettingsDialog::sizeHint() const
+{
+    // Take the font size into account for the window size, as we do for UI elements
+    const float fontSize = QFontDatabase::systemFont(QFontDatabase::GeneralFont).pointSizeF();
+    return QSize(qRound(58 * fontSize), qRound(62 * fontSize));
+}
+
 bool SettingsDialog::hasChanged()
 {
     return mShortcutsPage->isModified() || KConfigDialog::hasChanged();
