@@ -1,7 +1,7 @@
 /*
  *  SPDX-FileCopyrightText: 2015 Boudhayan Gupta <bgupta@kde.org>
  *
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #ifndef SETTINGSDIALOG_H
@@ -19,6 +19,10 @@ class SettingsDialog : public KConfigDialog
 
     explicit SettingsDialog(QWidget *parent = nullptr);
 
+    protected:
+
+    QSize sizeHint() const override;
+
     private:
 
     bool hasChanged() override;
