@@ -1,21 +1,26 @@
 /*
  *  SPDX-FileCopyrightText: 2018 Ambareesh "Amby" Balaji <ambareeshbalaji@gmail.com>
  *
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
+#include "QuickEditor.h"
+
+#include "ComparableQPoint.h"
+#include "settings.h"
+
 #include <KLocalizedString>
 #include <KWayland/Client/plasmashell.h>
 #include <KWayland/Client/surface.h>
 #include <KWindowSystem>
-#include <QGuiApplication>
-#include <QScreen>
-#include <QtCore/qmath.h>
-#include <QPainterPath>
-#include <QX11Info>
 
-#include "QuickEditor.h"
-#include "settings.h"
+#include <QGuiApplication>
+#include <QMouseEvent>
+#include <QPainter>
+#include <QPainterPath>
+#include <QScreen>
+#include <QtMath>
+#include <QX11Info>
 
 const int QuickEditor::handleRadiusMouse = 9;
 const int QuickEditor::handleRadiusTouch = 12;
@@ -42,8 +47,8 @@ const int QuickEditor::magZoom = 5;
 const int QuickEditor::magPixels = 16;
 const int QuickEditor::magOffset = 32;
 
-QuickEditor::QuickEditor(const QMap<ComparableQPoint, QImage> &images, KWayland::Client::PlasmaShell *plasmashell, QWidget *parent) :
-    QWidget(parent),
+QuickEditor::QuickEditor(const QMap<const QScreen *, QImage> &images, KWayland::Client::PlasmaShell *plasmashell, QWidget *parent) :
+    QWidget(parent, Qt::FramelessWindowHint | Qt::NoDropShadowWindowHint | Qt::Popup | Qt::WindowStaysOnTopHint),
     mMaskColor(QColor::fromRgbF(0, 0, 0, 0.15)),
     mStrokeColor(palette().highlight().color()),
     mCrossColor(QColor::fromRgbF(mStrokeColor.redF(), mStrokeColor.greenF(), mStrokeColor.blueF(), 0.7)),
@@ -66,7 +71,6 @@ QuickEditor::QuickEditor(const QMap<ComparableQPoint, QImage> &images, KWayland:
     mReleaseToCapture(Settings::useReleaseToCapture()),
     mRememberRegion(Settings::alwaysRememberRegion() || Settings::rememberLastRectangularRegion()),
     mDisableArrowKeys(false),
-    mPrimaryScreenGeo(QGuiApplication::primaryScreen()->geometry()),
     mbottomHelpLength(bottomHelpMaxLength),
     mHandleRadius(handleRadiusMouse)
 {
@@ -76,78 +80,14 @@ QuickEditor::QuickEditor(const QMap<ComparableQPoint, QImage> &images, KWayland:
 
     setMouseTracking(true);
     setAttribute(Qt::WA_StaticContents);
-    setWindowFlags(Qt::FramelessWindowHint | Qt::NoDropShadowWindowHint | Qt::Popup | Qt::WindowStaysOnTopHint);
 
     devicePixelRatio = plasmashell ? 1.0 : devicePixelRatioF();
     devicePixelRatioI = 1.0 / devicePixelRatio;
 
-    int width = 0, height = 0;
-    for (auto it = mImages.constBegin(); it != mImages.constEnd(); it ++) {
-        width = qMax(width, it.value().width() + it.key().x());
-        height = qMax(height, it.value().height() + it.key().y());
-    }
+    preparePaint();
+    createPixmapFromScreens();
+    setGeometryToScreenPixmap(plasmashell);
 
-    mPixmap = QPixmap(width, height);
-    mScreenRegion = QRegion();
-
-    const QList<QScreen*> screens = QGuiApplication::screens();
-    QMap<ComparableQPoint, QPair<qreal, QSize>> input;
-    for (auto it = mImages.begin(); it != mImages.end(); ++it) {
-        const auto pos = it.key();
-        const QImage &screenImage = it.value();
-        auto item = std::find_if(screens.constBegin(), screens.constEnd(),
-                                      [pos] (const QScreen* screen){
-            return screen->geometry().topLeft() == pos;
-        });
-        const QScreen* screen = *item;
-        input.insert(pos, QPair<qreal, QSize>(screenImage.width() / static_cast<qreal>(screen->size().width()), screenImage.size()));
-    }
-    const auto pointsTranslationMap = computeCoordinatesAfterScaling(input);
-    QPainter painter(&mPixmap);
-    for (auto it = mImages.constBegin(); it != mImages.constEnd(); it ++) {
-        painter.drawImage(pointsTranslationMap.value(it.key()), it.value());
-    }
-    painter.end();
-
-    if (KWindowSystem::isPlatformX11()) {
-        // Even though we want the quick editor window to be placed at (0, 0) in the native
-        // pixels, we cannot really specify a window position of (0, 0) if HiDPI support is on.
-        //
-        // The main reason for that is that Qt will scale the window position relative to the
-        // upper left corner of the screen where the quick editor is on in order to perform
-        // a conversion from the device-independent coordinates to the native pixels.
-        //
-        // Since (0, 0) in the device-independent pixels may not correspond to (0, 0) in the
-        // native pixels, we use XCB API to place the quick editor window at (0, 0).
-
-        uint16_t mask = 0;
-
-        mask |= XCB_CONFIG_WINDOW_X;
-        mask |= XCB_CONFIG_WINDOW_Y;
-
-        const uint32_t values[] = {
-            /* x */ 0,
-            /* y */ 0,
-        };
-
-        xcb_configure_window(QX11Info::connection(), winId(), mask, values);
-        resize(width, height);
-    } else {
-        setGeometry(0, 0, width, height);
-    }
-
-    // TODO This is a hack until a better interface is available
-    if (plasmashell) {
-        using namespace KWayland::Client;
-        winId();
-        auto surface = Surface::fromWindow(windowHandle());
-        if (surface) {
-            PlasmaShellSurface *plasmashellSurface = plasmashell->createSurface(surface, this);
-            plasmashellSurface->setRole(PlasmaShellSurface::Role::Panel);
-            plasmashellSurface->setPanelTakesFocus(true);
-            plasmashellSurface->setPosition(geometry().topLeft());
-        }
-    }
     if (Settings::rememberLastRectangularRegion() || Settings::alwaysRememberRegion()) {
         auto savedRect = Settings::cropRegion();
         QRect cropRegion = QRect(savedRect[0], savedRect[1], savedRect[2], savedRect[3]);
@@ -181,7 +121,6 @@ QuickEditor::QuickEditor(const QMap<ComparableQPoint, QImage> &images, KWayland:
     }
     layoutBottomHelpText();
 
-    preparePaint();
     update();
 }
 
@@ -211,18 +150,16 @@ void QuickEditor::acceptSelection()
 
             QPixmap output(mSelection.size() * maxDpr);
             QPainter painter(&output);
-            QRect intersected;
-            QPixmap screenOutput;
 
-            for (auto it = mRectToDpr.constBegin(); it != mRectToDpr.constEnd(); ++it)
-            {
-                const auto screenRect = (*it).first;
+            for (auto it = mScreenToDpr.constBegin(); it != mScreenToDpr.constEnd(); ++it) {
+                const QScreen *screen = it.key();
+                const QRect &screenRect = screen->geometry();
 
                 if (mSelection.intersects(screenRect)) {
-                    const auto &pos = screenRect.topLeft();
-                    const qreal &dpr = (*it).second;
+                    const ComparableQPoint pos = screenRect.topLeft();
+                    const qreal dpr = it.value();
 
-                    intersected = screenRect.intersected(mSelection);
+                    QRect intersected = screenRect.intersected(mSelection);
 
                     // converts to screen size & position
                     QRect pixelOnScreenIntersected;
@@ -230,30 +167,26 @@ void QuickEditor::acceptSelection()
                     pixelOnScreenIntersected.setWidth(intersected.width() * dpr);
                     pixelOnScreenIntersected.setHeight(intersected.height() * dpr);
 
-                    screenOutput = QPixmap::fromImage(mImages.value(pos).copy(pixelOnScreenIntersected));
+                    QPixmap screenOutput = QPixmap::fromImage(mImages.value(screen).copy(pixelOnScreenIntersected));
 
                     if (intersected.size() == mSelection.size()) {
-                        painter.end();
-
                         // short path when single screen
                         // keep native screen resolution
                         emit grabDone(screenOutput);
                         return;
 
-                    } else {
-
-                        // upscale the image according to max screen dpr, to keep the image not distorted
-                        const auto dprI = maxDpr / dpr;
-                        QBrush brush(screenOutput);
-                        brush.setTransform(QTransform().scale(dprI, dprI));
-                        intersected.moveTopLeft((intersected.topLeft() - mSelection.topLeft()) * maxDpr);
-                        intersected.setSize(intersected.size() * maxDpr);
-                        painter.setBrushOrigin(intersected.topLeft());
-                        painter.fillRect(intersected, brush);
                     }
+
+                    // upscale the image according to max screen dpr, to keep the image not distorted
+                    const auto dprI = maxDpr / dpr;
+                    QBrush brush(screenOutput);
+                    brush.setTransform(QTransform::fromScale(dprI, dprI));
+                    intersected.moveTopLeft((intersected.topLeft() - mSelection.topLeft()) * maxDpr);
+                    intersected.setSize(intersected.size() * maxDpr);
+                    painter.setBrushOrigin(intersected.topLeft());
+                    painter.fillRect(intersected, brush);
                 }
             }
-            painter.end();
 
             emit grabDone(output);
         }
@@ -422,10 +355,9 @@ void QuickEditor::mousePressEvent(QMouseEvent* event)
         * have mouse focus. So just grab it undconditionally here.
         */
         grabKeyboard();
-        const QPointF& pos = event->pos();
-        mMousePos = pos;
+        mMousePos = event->pos();
         mMagnifierAllowed = true;
-        mMouseDragState = mouseLocation(pos);
+        mMouseDragState = mouseLocation(mMousePos);
         mDisableArrowKeys = true;
         switch(mMouseDragState) {
         case MouseState::Outside:
@@ -465,12 +397,11 @@ void QuickEditor::mousePressEvent(QMouseEvent* event)
 
 void QuickEditor::mouseMoveEvent(QMouseEvent* event)
 {
-    const QPointF& pos = event->pos();
-    mMousePos = pos;
+    mMousePos = event->pos();
     mMagnifierAllowed = true;
     switch (mMouseDragState) {
     case MouseState::None: {
-        setMouseCursor(pos);
+        setMouseCursor(mMousePos);
         mMagnifierAllowed = false;
         break;
     }
@@ -478,46 +409,46 @@ void QuickEditor::mouseMoveEvent(QMouseEvent* event)
     case MouseState::TopRight:
     case MouseState::BottomRight:
     case MouseState::BottomLeft: {
-        const bool afterX = pos.x() >= mStartPos.x();
-        const bool afterY = pos.y() >= mStartPos.y();
+        const bool afterX = mMousePos.x() >= mStartPos.x();
+        const bool afterY = mMousePos.y() >= mStartPos.y();
         mSelection.setRect(
-            afterX ? mStartPos.x() : pos.x(),
-            afterY ? mStartPos.y() : pos.y(),
-            qAbs(pos.x() - mStartPos.x()) + (afterX ? devicePixelRatioI : 0),
-            qAbs(pos.y() - mStartPos.y()) + (afterY ? devicePixelRatioI : 0)
+            afterX ? mStartPos.x() : mMousePos.x(),
+            afterY ? mStartPos.y() : mMousePos.y(),
+            qAbs(mMousePos.x() - mStartPos.x()) + (afterX ? devicePixelRatioI : 0),
+            qAbs(mMousePos.y() - mStartPos.y()) + (afterY ? devicePixelRatioI : 0)
         );
         update();
         break;
     }
     case MouseState::Outside: {
         mSelection.setRect(
-            qMin(pos.x(), mStartPos.x()),
-            qMin(pos.y(), mStartPos.y()),
-            qAbs(pos.x() - mStartPos.x()) + devicePixelRatioI,
-            qAbs(pos.y() - mStartPos.y()) + devicePixelRatioI
+            qMin(mMousePos.x(), mStartPos.x()),
+            qMin(mMousePos.y(), mStartPos.y()),
+            qAbs(mMousePos.x() - mStartPos.x()) + devicePixelRatioI,
+            qAbs(mMousePos.y() - mStartPos.y()) + devicePixelRatioI
         );
         update();
         break;
     }
     case MouseState::Top:
     case MouseState::Bottom: {
-        const bool afterY = pos.y() >= mStartPos.y();
+        const bool afterY = mMousePos.y() >= mStartPos.y();
         mSelection.setRect(
             mSelection.x(),
-            afterY ? mStartPos.y() : pos.y(),
+            afterY ? mStartPos.y() : mMousePos.y(),
             mSelection.width(),
-            qAbs(pos.y() - mStartPos.y()) + (afterY ? devicePixelRatioI : 0)
+            qAbs(mMousePos.y() - mStartPos.y()) + (afterY ? devicePixelRatioI : 0)
         );
         update();
         break;
     }
     case MouseState::Right:
     case MouseState::Left: {
-        const bool afterX = pos.x() >= mStartPos.x();
+        const bool afterX = mMousePos.x() >= mStartPos.x();
         mSelection.setRect(
-            afterX ? mStartPos.x() : pos.x(),
+            afterX ? mStartPos.x() : mMousePos.x(),
             mSelection.y(),
-            qAbs(pos.x() - mStartPos.x()) + (afterX ? devicePixelRatioI : 0),
+            qAbs(mMousePos.x() - mStartPos.x()) + (afterX ? devicePixelRatioI : 0),
             mSelection.height()
         );
         update();
@@ -530,20 +461,18 @@ void QuickEditor::mouseMoveEvent(QMouseEvent* event)
         // in which case we adjust the diff to not let that happen
 
         // new top left point of the rectangle
-        QPoint newTopLeft = ((pos - mStartPos + mInitialTopLeft) * devicePixelRatio).toPoint();
+        QPoint newTopLeft = ((mMousePos - mStartPos + mInitialTopLeft) * devicePixelRatio).toPoint();
 
-        auto newRect = QRect(newTopLeft, mSelection.size() * devicePixelRatio);
+        const QRect newRect(newTopLeft, mSelection.size() * devicePixelRatio);
 
-        auto screenBoundingRect = mScreenRegion.boundingRect();
-        screenBoundingRect = QRect(screenBoundingRect.topLeft(), screenBoundingRect.size());
-        if (!screenBoundingRect.contains(newRect)) {
+        const QRect translatedScreensRect = mScreensRect.translated(-mScreensRect.topLeft());
+        if (!translatedScreensRect.contains(newRect)) {
             // Keep the item inside the scene screen region bounding rect.
-            newTopLeft.setX(qMin(screenBoundingRect.right() - newRect.width(), qMax(newTopLeft.x(), screenBoundingRect.left())));
-            newTopLeft.setY(qMin(screenBoundingRect.bottom() - newRect.height(), qMax(newTopLeft.y(), screenBoundingRect.top())));
+            newTopLeft.setX(qMin(translatedScreensRect.right() - newRect.width(), qMax(newTopLeft.x(), translatedScreensRect.left())));
+            newTopLeft.setY(qMin(translatedScreensRect.bottom() - newRect.height(), qMax(newTopLeft.y(), translatedScreensRect.top())));
         }
 
-        const auto newTopLeftF = newTopLeft * devicePixelRatioI;
-        mSelection.moveTo(newTopLeftF);
+        mSelection.moveTo(newTopLeft * devicePixelRatioI);
         update();
         break;
     }
@@ -556,19 +485,23 @@ void QuickEditor::mouseMoveEvent(QMouseEvent* event)
 
 void QuickEditor::mouseReleaseEvent(QMouseEvent* event)
 {
-    const auto button = event->button();
-    if (button == Qt::LeftButton) {
+    switch (event->button()) {
+    case Qt::LeftButton:
+        if (mMouseDragState == MouseState::Outside && mReleaseToCapture) {
+            acceptSelection();
+            return;
+        }
         mDisableArrowKeys = false;
         if (mMouseDragState == MouseState::Inside) {
             setCursor(Qt::OpenHandCursor);
-        } else if (mMouseDragState == MouseState::Outside && mReleaseToCapture) {
-            event->accept();
-            mMouseDragState = MouseState::None;
-            return acceptSelection();
         }
-    } else if (button == Qt::RightButton) {
+        break;
+    case Qt::RightButton:
         mSelection.setWidth(0);
         mSelection.setHeight(0);
+        break;
+    default:
+        break;
     }
     event->accept();
     mMouseDragState = MouseState::None;
@@ -583,7 +516,7 @@ void QuickEditor::mouseDoubleClickEvent(QMouseEvent* event)
     }
 }
 
-QMap<ComparableQPoint, ComparableQPoint> QuickEditor::computeCoordinatesAfterScaling(QMap<ComparableQPoint, QPair<qreal, QSize>> outputsRect)
+QMap<ComparableQPoint, ComparableQPoint> QuickEditor::computeCoordinatesAfterScaling(const QMap<ComparableQPoint, QPair<qreal, QSize>> &outputsRect)
 {
     QMap<ComparableQPoint, ComparableQPoint> translationMap;
 
@@ -592,9 +525,9 @@ QMap<ComparableQPoint, ComparableQPoint> QuickEditor::computeCoordinatesAfterSca
     }
 
     for (auto i = outputsRect.constBegin(); i != outputsRect.constEnd(); ++i) {
-        const auto p = i.key();
-        const auto size = i.value().second;
-        const auto dpr = i.value().first;
+        const ComparableQPoint p = i.key();
+        const QSize &size = i.value().second;
+        const double dpr = i.value().first;
         if (!qFuzzyCompare(dpr, 1.0)) {
             // must update all coordinates of next rects
             int newWidth = size.width();
@@ -627,27 +560,82 @@ QMap<ComparableQPoint, ComparableQPoint> QuickEditor::computeCoordinatesAfterSca
 
 void QuickEditor::preparePaint()
 {
-    auto screens = QGuiApplication::screens();
     for (auto i = mImages.constBegin(); i != mImages.constEnd(); ++i) {
-        QImage screenImage = i.value();
-        const auto &pos = i.key();
-
-        auto item = std::find_if(screens.constBegin(), screens.constEnd(),
-                                      [pos] (const QScreen* screen){
-            return screen->geometry().topLeft() == pos;
-        });
-        const QScreen* screen = *item;
+        const QScreen *screen = i.key();
+        const QImage &screenImage = i.value();
 
         const qreal dpr = screenImage.width() / static_cast<qreal>(screen->geometry().width());
-        mRectToDpr.append( QPair<QRect, qreal>(screen->geometry(), dpr));
+        mScreenToDpr.insert(screen, dpr);
 
         QRect virtualScreenRect;
         if (KWindowSystem::isPlatformX11()) {
-            virtualScreenRect = QRect(pos, screenImage.size());
+            virtualScreenRect = QRect(screen->geometry().topLeft(), screenImage.size());
         } else {
-            virtualScreenRect = QRect(pos, screenImage.size() / dpr);
+            virtualScreenRect = QRect(screen->geometry().topLeft(), screenImage.size() / dpr);
+        }
+        mScreensRect = mScreensRect.united(virtualScreenRect);
+    }
+}
+
+void QuickEditor::createPixmapFromScreens()
+{
+    const QList<QScreen*> screens = QGuiApplication::screens();
+    QMap<ComparableQPoint, QPair<qreal, QSize>> input;
+    for (auto it = mImages.constBegin(); it != mImages.constEnd(); ++it) {
+        const QScreen *screen = it.key();
+        const QImage &screenImage = it.value();
+        input.insert(screen->geometry().topLeft(), QPair<qreal, QSize>(screenImage.width() / static_cast<qreal>(screen->size().width()), screenImage.size()));
+    }
+    const auto pointsTranslationMap = computeCoordinatesAfterScaling(input);
+
+    // Geometry can have negative coordinates, so it is necessary to subtract the upper left point, because coordinates on the widget are counted from 0
+    mPixmap = QPixmap(mScreensRect.width(), mScreensRect.height());
+    QPainter painter(&mPixmap);
+    for (auto it = mImages.constBegin(); it != mImages.constEnd(); ++it) {
+        painter.drawImage(pointsTranslationMap.value(it.key()->geometry().topLeft()) - mScreensRect.topLeft(), it.value());
+    }
+}
+
+void QuickEditor::setGeometryToScreenPixmap(KWayland::Client::PlasmaShell *plasmashell)
+{
+    if (!KWindowSystem::isPlatformX11()) {
+        setGeometry(mScreensRect);
+    } else {
+        // Even though we want the quick editor window to be placed at (0, 0) in the native
+        // pixels, we cannot really specify a window position of (0, 0) if HiDPI support is on.
+        //
+        // The main reason for that is that Qt will scale the window position relative to the
+        // upper left corner of the screen where the quick editor is on in order to perform
+        // a conversion from the device-independent coordinates to the native pixels.
+        //
+        // Since (0, 0) in the device-independent pixels may not correspond to (0, 0) in the
+        // native pixels, we use XCB API to place the quick editor window at (0, 0).
+
+        uint16_t mask = 0;
+
+        mask |= XCB_CONFIG_WINDOW_X;
+        mask |= XCB_CONFIG_WINDOW_Y;
+
+        const uint32_t values[] = {
+            /* x */ 0,
+            /* y */ 0,
+        };
+
+        xcb_configure_window(QX11Info::connection(), winId(), mask, values);
+        resize(qRound(mScreensRect.width() / devicePixelRatio), qRound(mScreensRect.height() / devicePixelRatio));
+    }
+
+    // TODO This is a hack until a better interface is available
+    if (plasmashell) {
+        using namespace KWayland::Client;
+        winId();
+        auto surface = Surface::fromWindow(windowHandle());
+        if (surface) {
+            PlasmaShellSurface *plasmashellSurface = plasmashell->createSurface(surface, this);
+            plasmashellSurface->setRole(PlasmaShellSurface::Role::Panel);
+            plasmashellSurface->setPanelTakesFocus(true);
+            plasmashellSurface->setPosition(geometry().topLeft());
         }
-        mScreenRegion = mScreenRegion.united(virtualScreenRect);
     }
 }
 
@@ -659,29 +647,23 @@ void QuickEditor::paintEvent(QPaintEvent *event)
     painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
     painter.eraseRect(rect());
 
-    auto screens = QGuiApplication::screens();
     for (auto i = mImages.constBegin(); i != mImages.constEnd(); ++i) {
-        QImage screenImage = i.value();
-        const auto &pos = i.key();
+        const QImage &screenImage = i.value();
+        const QScreen *screen = i.key();
 
-        auto item = std::find_if(screens.constBegin(), screens.constEnd(),
-                                      [pos] (const QScreen* screen){
-            return screen->geometry().topLeft() == pos;
-        });
-        const QScreen* screen = *item;
-
-        const qreal dpr = screenImage.width() / static_cast<qreal>(screen->geometry().width());
+        QRect rectToDraw = screen->geometry().translated(-mScreensRect.topLeft());
+        const qreal dpr = screenImage.width() / static_cast<qreal>(rectToDraw.width());
         const qreal dprI = 1.0 / dpr;
 
         QBrush brush(screenImage);
+        brush.setTransform(QTransform::fromScale(dprI, dprI));
 
-        brush.setTransform(QTransform().scale(dprI, dprI));
-
-        painter.setBrushOrigin(screen->geometry().topLeft() / devicePixelRatio);
-
-        QRect rectToDraw = screen->geometry();
         rectToDraw.moveTopLeft(rectToDraw.topLeft() / devicePixelRatio);
-        rectToDraw.setSize(rectToDraw.size() * devicePixelRatio);
+        if (KWindowSystem::isPlatformWayland()) {
+            rectToDraw.setSize(rectToDraw.size() * devicePixelRatio);
+        }
+
+        painter.setBrushOrigin(rectToDraw.topLeft());
         painter.fillRect(rectToDraw, brush);
     }
 
@@ -737,8 +719,9 @@ void QuickEditor::layoutBottomHelpText()
         contentWidth = qMax(contentWidth, mBottomHelpGridLeftWidth + maxRightWidth + bottomHelpBoxPairSpacing);
         contentHeight += (i != bottomHelpMaxLength ? bottomHelpBoxMarginBottom : 0);
     }
-    mBottomHelpContentPos.setX((mPrimaryScreenGeo.width() - contentWidth) / 2 + mPrimaryScreenGeo.x() / devicePixelRatio);
-    mBottomHelpContentPos.setY((mPrimaryScreenGeo.height() + mPrimaryScreenGeo.y() / devicePixelRatio) - contentHeight - 8 );
+    const QRect primaryGeometry = QGuiApplication::primaryScreen()->geometry().translated(-mScreensRect.topLeft());
+    mBottomHelpContentPos.setX((primaryGeometry.width() - contentWidth) / 2 + primaryGeometry.x() / devicePixelRatio);
+    mBottomHelpContentPos.setY((primaryGeometry.height() + primaryGeometry.y() / devicePixelRatio) - contentHeight - 8 );
     mBottomHelpGridLeftWidth += mBottomHelpContentPos.x();
     mBottomHelpBorderBox.setRect(
         mBottomHelpContentPos.x() - bottomHelpBoxPaddingX,
@@ -820,19 +803,19 @@ void QuickEditor::drawDragHandles(QPainter &painter)
     if (minEdgeLength < minDragHandleSpace) {
         offset = (minDragHandleSpace - minEdgeLength) / 2.0;
     } else {
-        QRect virtualScreenGeo = QGuiApplication::primaryScreen()->virtualGeometry();
+        const QRect translatedScreensRect = mScreensRect.translated(-mScreensRect.topLeft());
         const int penWidth = painter.pen().width();
 
-        offsetTop = top - virtualScreenGeo.top() - mHandleRadius;
+        offsetTop = top - translatedScreensRect.top() - mHandleRadius;
         offsetTop = (offsetTop >= 0) ? 0 : offsetTop;
 
-        offsetRight =  virtualScreenGeo.right() - right - mHandleRadius + penWidth;
+        offsetRight =  translatedScreensRect.right() - right - mHandleRadius + penWidth;
         offsetRight = (offsetRight >= 0) ? 0 : offsetRight;
 
-        offsetBottom = virtualScreenGeo.bottom() - bottom - mHandleRadius + penWidth;
+        offsetBottom = translatedScreensRect.bottom() - bottom - mHandleRadius + penWidth;
         offsetBottom = (offsetBottom >= 0) ? 0 : offsetBottom;
 
-        offsetLeft = left - virtualScreenGeo.left() - mHandleRadius;
+        offsetLeft = left - translatedScreensRect.left() - mHandleRadius;
         offsetLeft = (offsetLeft >= 0) ? 0 : offsetLeft;
     }
 
@@ -857,7 +840,7 @@ void QuickEditor::drawDragHandles(QPainter &painter)
     QPainterPath path;
 
     // add handles to the path
-    for (const QPointF &handlePosition : this->mHandlePositions) {
+    for (QPointF handlePosition : qAsConst(mHandlePositions)) {
         path.addEllipse(handlePosition, mHandleRadius, mHandleRadius);
     }
 
@@ -923,8 +906,9 @@ void QuickEditor::drawMidHelpText(QPainter &painter)
     painter.fillRect(rect(), mMaskColor);
     painter.setFont(mMidHelpTextFont);
     QRect textSize = painter.boundingRect(QRect(), Qt::AlignCenter, mMidHelpText);
-    QPoint pos((mPrimaryScreenGeo.width() - textSize.width()) / 2 + mPrimaryScreenGeo.x() / devicePixelRatio,
-               (mPrimaryScreenGeo.height() - textSize.height()) / 2 + mPrimaryScreenGeo.y() / devicePixelRatio);
+    const QRect primaryGeometry = QGuiApplication::primaryScreen()->geometry().translated(-mScreensRect.topLeft());
+    QPoint pos((primaryGeometry.width() - textSize.width()) / 2 + primaryGeometry.x() / devicePixelRatio,
+               (primaryGeometry.height() - textSize.height()) / 2 + primaryGeometry.y() / devicePixelRatio);
 
     painter.setBrush(mLabelBackgroundColor);
     QPen pen(mLabelForegroundColor);
@@ -1013,19 +997,26 @@ QuickEditor::MouseState QuickEditor::mouseLocation(const QPointF& pos)
 
     if (isPointInsideCircle(mHandlePositions[0], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::TopLeft;
-    } else if (isPointInsideCircle(mHandlePositions[1], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[1], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::TopRight;
-    } else if (isPointInsideCircle(mHandlePositions[2], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[2], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::BottomRight;
-    } else if (isPointInsideCircle(mHandlePositions[3], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[3], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::BottomLeft;
-    } else if (isPointInsideCircle(mHandlePositions[4], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[4], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::Top;
-    } else if (isPointInsideCircle(mHandlePositions[5], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[5], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::Right;
-    } else if (isPointInsideCircle(mHandlePositions[6], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[6], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::Bottom;
-    } else if (isPointInsideCircle(mHandlePositions[7], mHandleRadius * increaseDragAreaFactor, pos)) {
+    }
+    if (isPointInsideCircle(mHandlePositions[7], mHandleRadius * increaseDragAreaFactor, pos)) {
         return MouseState::Left;
     }
 
@@ -1042,21 +1033,22 @@ QuickEditor::MouseState QuickEditor::mouseLocation(const QPointF& pos)
         if (inRange(mSelection.x(), mSelection.x() + mSelection.width(), pos.x())) {
             if (withinThreshold(pos.y() - mSelection.y(), borderDragAreaSize)) {
                 return MouseState::Top;
-            } else if (withinThreshold(pos.y() - mSelection.y() - mSelection.height(), borderDragAreaSize)) {
+            }
+            if (withinThreshold(pos.y() - mSelection.y() - mSelection.height(), borderDragAreaSize)) {
                 return MouseState::Bottom;
             }
         }
         if (inRange(mSelection.y(), mSelection.y() + mSelection.height(), pos.y())) {
             if (withinThreshold(pos.x() - mSelection.x(), borderDragAreaSize)) {
                 return MouseState::Left;
-            } else if (withinThreshold(pos.x() - mSelection.x() - mSelection.width(), borderDragAreaSize)) {
+            }
+            if (withinThreshold(pos.x() - mSelection.x() - mSelection.width(), borderDragAreaSize)) {
                 return MouseState::Right;
             }
         }
     }
     if (mSelection.contains(pos.toPoint())) {
         return MouseState::Inside;
-    } else {
-        return MouseState::Outside;
     }
+    return MouseState::Outside;
 }
