@@ -1,40 +1,37 @@
 /*
  *  SPDX-FileCopyrightText: 2018 Ambareesh "Amby" Balaji <ambareeshbalaji@gmail.com>
  *
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #ifndef QUICKEDITOR_H
 #define QUICKEDITOR_H
 
-#include <QKeyEvent>
-#include <QPainter>
+#include <QMap>
 #include <QStaticText>
 #include <QWidget>
-#include <utility>
-#include <vector>
-
-#include "ComparableQPoint.h"
 
+class ComparableQPoint;
+class QKeyEvent;
 class QMouseEvent;
+class QPainter;
 
-namespace KWayland {
-namespace Client {
+namespace KWayland::Client {
 class PlasmaShell;
 }
-}
 
 class QuickEditor: public QWidget
 {
     Q_OBJECT
 
-    public:
+public:
+    explicit QuickEditor(const QMap<const QScreen *, QImage> &images, KWayland::Client::PlasmaShell *plasmashell, QWidget *parent = nullptr);
 
-    explicit QuickEditor(const QMap<ComparableQPoint, QImage> &images, KWayland::Client::PlasmaShell *plasmashell, QWidget *parent = nullptr);
-    virtual ~QuickEditor() = default;
-
-    private:
+Q_SIGNALS:
+    void grabDone(const QPixmap &thePixmap);
+    void grabCancelled();
 
+private:
     enum MouseState : short {
         None = 0, // 0000
         Inside = 1 << 0, // 0001
@@ -69,12 +66,17 @@ class QuickEditor: public QWidget
     void drawDragHandles(QPainter& painter);
     void drawMagnifier(QPainter& painter);
     void drawMidHelpText(QPainter& painter);
+    void preparePaint();
+    void createPixmapFromScreens();
+    void setGeometryToScreenPixmap(KWayland::Client::PlasmaShell *plasmashell);
     void drawSelectionSizeTooltip(QPainter& painter, bool dragHandlesVisible);
     void setBottomHelpText();
     void layoutBottomHelpText();
     void setMouseCursor(const QPointF& pos);
     MouseState mouseLocation(const QPointF& pos);
 
+    static QMap<ComparableQPoint, ComparableQPoint> computeCoordinatesAfterScaling(const QMap<ComparableQPoint, QPair<qreal, QSize>> &outputsRect);
+
     static const int handleRadiusMouse;
     static const int handleRadiusTouch;
     static const qreal increaseDragAreaFactor;
@@ -117,8 +119,8 @@ class QuickEditor: public QWidget
     QPoint mBottomHelpContentPos;
     int mBottomHelpGridLeftWidth;
     MouseState mMouseDragState;
-    QMap<ComparableQPoint, QImage> mImages;
-    QVector<QPair<QRect, qreal>> mRectToDpr;
+    QMap<const QScreen *, QImage> mImages;
+    QMap<const QScreen *, qreal> mScreenToDpr;
     QPixmap mPixmap;
     qreal devicePixelRatio;
     qreal devicePixelRatioI;
@@ -129,25 +131,13 @@ class QuickEditor: public QWidget
     bool mReleaseToCapture;
     bool mRememberRegion;
     bool mDisableArrowKeys;
-    QRect mPrimaryScreenGeo;
     int mbottomHelpLength;
-    QRegion mScreenRegion;
+    QRect mScreensRect;
 
     // Midpoints of handles
     QVector<QPointF> mHandlePositions = QVector<QPointF> {8};
     // Radius of handles is either handleRadiusMouse or handleRadiusTouch
     int mHandleRadius;
-
-Q_SIGNALS:
-
-    void grabDone(const QPixmap &thePixmap);
-    void grabCancelled();
-
-private:
-
-    QMap<ComparableQPoint, ComparableQPoint> computeCoordinatesAfterScaling(QMap<ComparableQPoint, QPair<qreal, QSize>> outputsRect);
-
-    void preparePaint();
 };
 
 #endif // QUICKEDITOR_H
