@@ -2,7 +2,7 @@
  *  SPDX-FileCopyrightText: 2019 David Redondo <kde@david-redondo.de>
  *  SPDX-FileCopyrightText: 2015 Boudhayan Gupta <bgupta@kde.org>
  *
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "SpectacleCore.h"
@@ -131,12 +131,16 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString& /*
     switch (mStartMode) {
 
     case StartMode::DBus:
-        mCopyToClipboard = Settings::copyImageToClipboard();
+        mCopyImageToClipboard = Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyImage;
+        mCopyLocationToClipboard = Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyLocation;
+
         qApp->setQuitOnLastWindowClosed(false);
         break;
 
     case StartMode::Background: {
-        mCopyToClipboard = false;
+        mCopyImageToClipboard = false;
+        mCopyLocationToClipboard = false;
+
         if (parser->isSet(QStringLiteral("nonotify"))) {
             mNotify = false;
         }
@@ -164,8 +168,10 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString& /*
             lDelayMsec = -1;
         }
 
-        if (parser->isSet(QStringLiteral("clipboard"))) {
-            mCopyToClipboard = true;
+        if (parser->isSet(QStringLiteral("copy-image"))) {
+            mCopyImageToClipboard = true;
+        } else if (parser->isSet(QStringLiteral("copy-path"))) {
+            mCopyLocationToClipboard = true;
         }
 
         if (!mIsGuiInited) {
@@ -229,6 +235,8 @@ void SpectacleCore::setUpShortcuts()
 
     KGlobalAccel::self()->setGlobalShortcut(ShortcutActions::self()->activeWindowAction(), Qt::META | Qt::Key_Print);
 
+    KGlobalAccel::self()->setGlobalShortcut(ShortcutActions::self()->windowUnderCursorAction(), Qt::META | Qt::CTRL | Qt::Key_Print);
+
     KGlobalAccel::self()->setGlobalShortcut(ShortcutActions::self()->regionAction(), Qt::META | Qt::SHIFT | Qt::Key_Print);
 
     KGlobalAccel::self()->setGlobalShortcut(ShortcutActions::self()->currentScreenAction(), QList<QKeySequence>());
@@ -281,7 +289,7 @@ void SpectacleCore::showErrorMessage(const QString &theErrString)
 
 void SpectacleCore::screenshotsUpdated(const QVector<QImage> &imgs)
 {
-    QMap<ComparableQPoint, QImage> mapScreens;
+    QMap<const QScreen *, QImage> mapScreens;
     QList<QScreen *> screens = QGuiApplication::screens();
 
     if (imgs.length() != screens.size()) {
@@ -292,10 +300,7 @@ void SpectacleCore::screenshotsUpdated(const QVector<QImage> &imgs)
     // only used by Spectacle::CaptureMode::RectangularRegion
     auto it = imgs.constBegin();
     for (const QScreen *screen: screens) {
-        auto pos = screen->geometry().topLeft();
-        auto img = *it;
-        auto dpr = img.width() / static_cast<qreal>(screen->geometry().width());
-        mapScreens.insert(pos, img);
+        mapScreens.insert(screen, *it);
         ++it;
     }
 
@@ -323,22 +328,24 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
     case StartMode::Background:
     case StartMode::DBus:
         {
-            if (mSaveToOutput || !mCopyToClipboard || (Settings::autoSaveImage() && !mSaveToOutput)) {
+            if (mSaveToOutput || !mCopyImageToClipboard || (Settings::autoSaveImage() && !mSaveToOutput)) {
                 mSaveToOutput = Settings::autoSaveImage();
                 QUrl lSavePath = (mStartMode == StartMode::Background && mFileNameUrl.isValid() && mFileNameUrl.isLocalFile()) ?
                     mFileNameUrl : QUrl();
                 lExportManager->doSave(lSavePath, mNotify);
             }
 
-            if (mCopyToClipboard) {
+            if (mCopyImageToClipboard) {
                 lExportManager->doCopyToClipboard(mNotify);
+            } else if (mCopyLocationToClipboard) {
+                lExportManager->doCopyLocationToClipboard(mNotify);
             }
 
             // if we don't have a Gui already opened, emit allDone
             if (!this->mIsGuiInited) {
                 // if we notify, we emit allDone only if the user either dismissed the notification or pressed
                 // the "Open" button, otherwise the app closes before it can react to it.
-                if (!mNotify && mCopyToClipboard) {
+                if (!mNotify && mCopyImageToClipboard) {
                     // Allow some time for clipboard content to transfer if '--nonotify' is used, see Bug #411263
                     // TODO: Find better solution
                     QTimer::singleShot(250, this, &SpectacleCore::allDone);
@@ -352,7 +359,8 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
         mMainWindow->setScreenshotAndShow(thePixmap);
 
         bool autoSaveImage = Settings::autoSaveImage();
-        bool mCopyImageToClipboard = Settings::copyImageToClipboard();
+        bool mCopyImageToClipboard = Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyImage;
+        bool mCopyLocationToClipboard = Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyLocation;
 
         if (autoSaveImage && mCopyImageToClipboard) {
             lExportManager->doSaveAndCopy();
@@ -360,6 +368,8 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
             lExportManager->doSave();
         } else if (mCopyImageToClipboard) {
             lExportManager->doCopyToClipboard(false);
+        } else if (mCopyLocationToClipboard) {
+            lExportManager->doCopyLocationToClipboard(false);
         }
     }
 }
@@ -429,7 +439,7 @@ void SpectacleCore::doNotify(const QUrl &theSavedAt)
         lNotify->setText(i18n("A screenshot was saved as '%1' to '%2'.", theSavedAt.fileName(), lSavePath));
         // set to false so it won't show the same message twice
         mSaveToOutput = false;
-    } else if (mCopyToClipboard) {
+    } else if (mCopyImageToClipboard) {
         lNotify->setText(i18n("A screenshot was saved to your clipboard."));
     } else if (lSavePath == QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)) {
         lNotify->setText(i18nc("Placeholder is filename", "A screenshot was saved as '%1' to your Pictures folder.", theSavedAt.fileName()));
@@ -464,7 +474,7 @@ void SpectacleCore::doNotify(const QUrl &theSavedAt)
 
 void SpectacleCore::doCopyPath(const QUrl &savedAt)
 {
-    if (Settings::copySaveLocation()) {
+    if (Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyLocation) {
         qApp->clipboard()->setText(savedAt.toLocalFile());
     }
 }
@@ -484,7 +494,8 @@ void SpectacleCore::populateCommandLineParser(QCommandLineParser *lCmdLineParser
         {{QStringLiteral("n"), QStringLiteral("nonotify")},          i18n("In background mode, do not pop up a notification when the screenshot is taken")},
         {{QStringLiteral("o"), QStringLiteral("output")},            i18n("In background mode, save image to specified file"), QStringLiteral("fileName")},
         {{QStringLiteral("d"), QStringLiteral("delay")},             i18n("In background mode, delay before taking the shot (in milliseconds)"), QStringLiteral("delayMsec")},
-        {{QStringLiteral("c"), QStringLiteral("clipboard")},         i18n("In background mode, copy screenshot to clipboard")},
+        {{QStringLiteral("c"), QStringLiteral("copy-image")},        i18n("In background mode, copy screenshot image to clipboard")},
+        {{QStringLiteral("C"), QStringLiteral("copy-path")},         i18n("In background mode, copy screenshot file path to clipboard")},
         {{QStringLiteral("w"), QStringLiteral("onclick")},           i18n("Wait for a click before taking screenshot. Invalidates delay")},
         {{QStringLiteral("i"), QStringLiteral("new-instance")},      i18n("Starts a new GUI instance of spectacle without registering to DBus")},
         {{QStringLiteral("p"), QStringLiteral("pointer")},           i18n("In background mode, include pointer in the screenshot")},
